{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "20bbe2cd-fc03-402d-b42c-06bf9a653f02",
   "metadata": {},
   "source": [
    "# Simulation Set-up\n",
    "\n",
    "In this section a simulation set-up is going to be conducted. The simulation set-up in <em>Loads Kernel</em> is going to be shown in detail step by step. The <em>Phyton</em> file used for this notebook is named 'jcl_dc3_trim.py' and can be found in the folder tutorials -> DC3_model -> JCLs. To start the libraries needed are imported. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "110d88e0-4304-49cd-b545-597b2198b450",
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "import platform\n",
    "import os\n",
    "from loadskernel.units import ft2m, tas2Ma\n",
    "from loadskernel import jcl_helper\n",
    "import pathlib"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "30e69626-3527-471e-9eea-bf45e383f93f",
   "metadata": {},
   "source": [
    "In the second step of the set-up simulation process, a name for the aircarft and some general parameters need to be assigned. This parameters are essential to then compute force and moment coefficients, as well as the stability derivatives."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "2c2e8223-fa46-4a29-8ce7-518e13d871d4",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        model_root = pathlib.Path(__file__).parent.parent.resolve()\n",
    "\n",
    "        # Give your aircraft a name and set some general parameters\n",
    "        self.general = {'aircraft': 'DC3',\n",
    "                        # Reference span width (from tip to tip)\n",
    "                        'b_ref': 29.0,\n",
    "                        # Reference chord length\n",
    "                        'c_ref': 3.508,\n",
    "                        # Reference area\n",
    "                        'A_ref': 91.7,\n",
    "                        # Mean aerodynamic center, also used as moments reference point\n",
    "                        'MAC_ref': [8.566, 0.0, 0.0],\n",
    "                        }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "4348eb8c-4f9b-4f2a-8f50-8deeb1cdf173",
   "metadata": {},
   "source": [
    "Afterwards, the eletronic flight control system is set-up. The electronic flight control system (EFCS) provides the \"wireing\" of the pilot \n",
    "commands xi, eta and zeta with the control surface deflections. This is aircraft specific and needs to be implemented as a python module."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "60f76302-fd1c-47bb-aeab-2aa83e952b5f",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "        \n",
    "        self.efcs = {'version': 'efcs_dc3', # Name of the corresponding python module\n",
    "                     # Path where to find the EFCS module\n",
    "                     'path': os.path.join(model_root, 'efcs'),\n",
    "                     }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "a39cfaed-f838-4549-b6ac-0666f7a8d510",
   "metadata": {},
   "source": [
    "The next step is to read the structural geometry. This consists in reading the bdf file(s) with GRIDs, RBE2s, CBARs, PBARs, CONM2s, MATs and CORDs (CORD1R and CORD2R) cards; the file(s) that contains the monitoring stations; and, to finalize, it reads the mass and stiffness matrices, obtained using <em>MSC Nastran</em>. Important to refer that all these files correpond to the structure only configuration, hence no payload, fuel or system masses. For more information about the <em>MSC Nastran</em> cards please refer to {cite}`Nastran`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "c6613729-8d49-4f14-a4f8-200b922f7c40",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.geom = {'method': 'mona',  # ModGen and/or Nastran (mona) BDFs\n",
    "                     # bdf file(s) with GRIDs and CORDs (CORD1R and CORD2R)\n",
    "                     'filename_grid': [os.path.join(model_root, 'fem', 'structure_only.bdf')],\n",
    "                     # bdf file(s) with CQUADs and CTRIAs, for visualization only, e.g. outer skin on the aircraft\n",
    "                     # 'filename_shell': [],\n",
    "                     # bdf file(s) with MONPNT-cards\n",
    "                     'filename_monpnt': os.path.join(model_root, 'fem', 'export_monitoring-stations.csv'),\n",
    "                     # The following matrices are required for some mass methods. However, the stiffness is geometry\n",
    "                     # and not mass dependent. Overview:\n",
    "                     # KGG via DMAP Alter und OP4            - required for mass method = 'modalanalysis', 'guyan' or 'B2000'\n",
    "                     # USET via DMAP Alter und OP4           - required for mass method = 'modalanalysis', 'guyan'\n",
    "                     # matrix GM via DMAP Alter und OP4      - required for mass method = 'modalanalysis', 'guyan'\n",
    "                     # bdf file(s) with ASET1-card           - required for mass method = 'guyan'\n",
    "                     # matrix R_trans frum B2000             - required for mass method = 'B2000'\n",
    "                     'filename_h5': os.path.join(model_root, 'fem', 'SOL103_structure_only.mtx.h5'),\n",
    "                     'filename_uset': os.path.join(model_root, 'fem', 'uset.op2'),\n",
    "                     }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "ef4aaf95-a834-4294-a787-2eca1174925e",
   "metadata": {},
   "source": [
    "Following the structure model, the aerodynamic model needs to be set-up. The set-up starts with the selection of simulation aerodynamic method. In this specific simulation case the 'mona_steady' method was used. Nevertheless, five more methods are available (for more information refer to {cite}`vos_loads_2020`). Then, true or false is chosen for the field 'flex'. Activating this 'flex' mode means activating the aerodynamic feedback of elastic structure on aerodynamics. In the case of desactivation of this 'flex' mode deformation are still visible, but there is no coupling between elastic structure deformation and aerodynamic forces. Furthermore, the aerodynamic grid is read. This process is done reading the file(s) containing the CAERO1s (or CAERO7s, or CQUAD4s) , AESURFs and AELISTs cards. Also, the DMI matrix for camber and twist corrrection is read. Moreover, the hingeline of the control surfaces is defined, followed by the selection of the aerodynamic model. In the case of the flutter analyses the Doublet-Lattice Method (DLM) is chosen. After selecting the aerodynamic model, a reference velocity(ies) and a serie of reduced frequencies are descriminated for the DLM calculations. At the end, the number of poles for Rational Function Approximation (RFA) is specified."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "c132bace-23e0-440a-992b-11aa77196b27",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.aero = {'method': 'mona_steady',\n",
    "                     # 'mona_steady'      - steady trim and quasi-steady time domain simulations\n",
    "                     # 'mona_unsteady'    - unsteady time domain simulation based on the RFA, e.g. for gust\n",
    "                     # 'freq_dom'         - frequency domain simulations, e.g. gust, continuous turbulence, flutter, etc\n",
    "                     # 'nonlin_steady'    - steady trim and quasi-steady time domain simulations with some non-linearities\n",
    "                     # 'cfd_steady'       - steady trim\n",
    "                     # 'cfd_unsteady'     - unsteady time domain simulation, e.g. for gust\n",
    "                     #\n",
    "                     # True or False, aerodynamic feedback of elastic structure on aerodynamics can be deactivated.\n",
    "                     # You will still see deformations, but there is no coupling.\n",
    "                     'flex': True,\n",
    "                     # aerogrid is given by CAERO1, CAERO7 or by CQUAD4 cards\n",
    "                     'method_caero': 'CAERO1',\n",
    "                     # bdf file(s) with CAERO1 or CQUAD4-cards for aerogrid. IDs in ascending order.\n",
    "                     'filename_caero_bdf': [os.path.join(model_root, 'aero', 'vt',  'vt.CAERO1'),\n",
    "                                            os.path.join(model_root, 'aero', 'left-ht', 'left-ht.CAERO1'),\n",
    "                                            os.path.join(model_root, 'aero', 'right-ht', 'right-ht.CAERO1'),\n",
    "                                            os.path.join(model_root, 'aero', 'left-wing', 'left-wing.CAERO1'),\n",
    "                                            os.path.join(model_root, 'aero', 'right-wing', 'right-wing.CAERO1'),\n",
    "                                            ],\n",
    "                     # DMI Matrix for camber and twist correction. Same order as the aerogrid.\n",
    "                     'filename_DMI_W2GJ': [os.path.join(model_root, 'fem', 'w2gj_list.DMI_merge')],\n",
    "                     # bdf file(s) with AESURF-cards\n",
    "                     'filename_aesurf': [os.path.join(model_root, 'aero','vt', 'vt.AESURF'),\n",
    "                                         os.path.join(model_root, 'aero', 'left-ht','left-ht.AESURF'),\n",
    "                                         os.path.join(model_root, 'aero', 'right-ht', 'right-ht.AESURF'),\n",
    "                                         os.path.join(model_root, 'aero', 'left-wing','left-wing.AESURF'),\n",
    "                                         os.path.join(model_root, 'aero', 'right-wing','right-wing.AESURF'),\n",
    "                                         ],\n",
    "                     # bdf file(s) with AELIST-cards\n",
    "                     'filename_aelist': [os.path.join(model_root, 'aero', 'vt', 'vt.AELIST'),\n",
    "                                         os.path.join(model_root, 'aero', 'left-ht', 'left-ht.AELIST'),\n",
    "                                         os.path.join(model_root, 'aero', 'right-ht', 'right-ht.AELIST'),\n",
    "                                         os.path.join(model_root, 'aero', 'left-wing','left-wing.AELIST'),\n",
    "                                         os.path.join(model_root, 'aero', 'right-wing','right-wing.AELIST'),\n",
    "                                         ],\n",
    "                     # The hingeline of a CS is given by a CORD. Either the y- or the z-axis is taken as hingeline. 'y', 'z'\n",
    "                     'hingeline': 'y',\n",
    "                     # 'vlm' (panel-aero), 'dlm' (panel-aero) or 'nastran' (external form matrices)\n",
    "                     'method_AIC': 'vlm',\n",
    "                     'key': ['VC', 'VD'],\n",
    "                     'Ma': [0.27, 0.34],\n",
    "                     }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "b8908798-5abf-4627-81d1-5c1620c8ee76",
   "metadata": {},
   "source": [
    "Splining in aeroelastic analyses is a crucial technique for accurately tranferring data between structural and aerodynamic model, ensuring precise and stable simulations of the interactions between aerodynamic forces and structural deformations. To set the way in which the aerodynamic forces are applied to the structure the following code lines are used. For that, the method needs to be chosen (for more information about the different methods please refer to {cite}`vos_loads_2020`); then, there is the possibility to use only a subset of the structural grid for splinning; lastly, the bdf file(s) with GRIDs to be used as the subset is called."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "675e8c90-6e86-4e72-bc2c-a5e65534272c",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.spline = {'method': 'nearest_neighbour',  # Options: 'nearest_neighbour', 'rbf' or 'nastran'\n",
    "                       # Possibility to use only a subset of the structural grid for splining. True or False\n",
    "                       'splinegrid': False,\n",
    "                       # bdf file(s) with GRIDs to ne used\n",
    "                       'filename_splinegrid': ['splinegrid.bdf']\n",
    "                       }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "234f3365-0435-4176-9580-7ba1fa31e3f8",
   "metadata": {},
   "source": [
    "The next step is to specify the settings for the structural dynamics. In the case of the DC-3 model the method chosen was 'modalanalyses' (for more information about the methods please refer to {cite}`vos_loads_2020`). Afterwards, the file .h5 from <em>MSC Nastran</em> SOL103 is called. To finalize there is an option to omit the first six modes (the rigid body modes), and the list(s) of modes to use."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "3e5d5af6-fc6c-43d8-af87-144890972dd1",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.mass = {'method': 'modalanalysis', # Inplemented interfaces: 'f06', 'modalanalysis', 'guyan', 'CoFE', 'B2000'\n",
    "                     'key': ['M3'],\n",
    "                     # MGG via DMAP Alter and OP4 - always required\n",
    "                     'filename_h5': [\n",
    "                                     os.path.join(model_root, 'fem', 'SOL103_M3.mtx.h5'),\n",
    "                                     ],\n",
    "                     # True or False, omits first six modes\n",
    "                     'omit_rb_modes': True,\n",
    "                     # list(s) of modes to use\n",
    "                     'modes': [np.arange(1, 71), np.arange(1, 71), np.arange(1, 71), np.arange(1, 71)],\n",
    "                     }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "df904e08-46cc-48e8-be0a-90e250c194d7",
   "metadata": {},
   "source": [
    "Thereafter, the modal damping can be applied as a factor of the stiffness matrix. As well, the list of altitudes is discriminated and the method for the rigid body equations of motion is selected."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "c437d35e-3702-4732-ac70-5c5903a586a4",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.damping = {'method': 'modal',\n",
    "                        'damping': 0.02,\n",
    "                        }\n",
    "        self.atmo = {'method': 'ISA',\n",
    "                     'key': ['FL000', 'FL055', 'FL075', 'FL210'],\n",
    "                     # Altitude in meters\n",
    "                     'h': ft2m([0, 5500, 7500, 21000,]),\n",
    "                     }\n",
    "        self.eom = {'version': 'waszak'}  # 'linear' or 'waszak'"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "bdfd4939-467c-4b9b-a4a6-da6ae85f46b1",
   "metadata": {},
   "source": [
    "This section controls the automatic plotting and selection of dimensioning load cases. Simply put a list of names of the monitoring stations into the dictionary of possible load plots listed below. This will generate a pdf document and <em>MSC Nastran</em> force and moment cards for the dimensioning load cases.  "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "1da8cee2-3d1d-47e6-8e01-84856c93f504",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.loadplots = {'potatos_fz_mx': [],\n",
    "                          'potatos_mx_my': ['WL01','WL03','WL05','WL07','WL09','WL11','WL13','WL15','WL17','WL19','WL21','WL23','WL25','WL27','WL29','WL31','WR31','WR29','WR27','WR25','WR23','WR21','WR19','WR17','WR15','WR13','WR11','WR09','WR07','WR05','WR03','WR01'],\n",
    "                          'potatos_fz_my': [],\n",
    "                          'potatos_fy_mx': [],\n",
    "                          'potatos_mx_mz': [],\n",
    "                          'potatos_my_mz': [],\n",
    "                          'cuttingforces_wing': ['WL01','WL03','WL05','WL07','WL09','WL11','WL13','WL15','WL17','WL19','WL21','WL23','WL25','WL27','WL29','WL31','WR31','WR29','WR27','WR25','WR23','WR21','WR19','WR17','WR15','WR13','WR11','WR09','WR07','WR05','WR03','WR01'],\n",
    "                          }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8e6a552c-4b54-4c5b-a527-1a028a6c7623",
   "metadata": {},
   "source": [
    "The trimcase defines the maneuver load case, one dictionary per load case. There may be hundreds or thousands of load cases, so at some point it might be beneficial to script this section or import an excel sheet. Here, for the specific case of this simulation three different trim cases were chosen. The first one correspond to horizontal level flight (n<sub>Z</sub>=1) at 70 m/s TAS and FL000. The second case concerns a push-down maneuver (n<sub>Z</sub>=-1) at the same velocity and altitude conditions. At last, a pull-up manuever (n<sub>Z</sub>=2.5) at the same velocity and altitude conditions is set. For the three cases the mass configuration M3 was chosen."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "adc94672-0e88-4eba-9e85-4c3faed56dd0",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.trimcase = [{'desc': 'CC.M3.OVCFL000.level', # Descriptive string of the maneuver case\n",
    "                          # Kind of trim condition, blank for trim about all three axes, for more trim conditions see\n",
    "                          # trim_conditions.py\n",
    "                          'maneuver': '',\n",
    "                          # Subcase ID number, for Nastran in acending order\n",
    "                          'subcase': 1,\n",
    "                          # Setting of the operational point\n",
    "                          # The flight speed is given by the Mach number\n",
    "                          'Ma': tas2Ma(70.0, 0.0),\n",
    "                          # Aero key\n",
    "                          'aero': 'VC',\n",
    "                          # Atmo key\n",
    "                          'altitude': 'FL000',\n",
    "                          # Mass key\n",
    "                          'mass': 'M3',\n",
    "                          # Load factor Nz\n",
    "                          'Nz': 1.0,\n",
    "                          # Velocities and accelerations given in ISO 9300 coordinate system (right-handed, forward-right-down)\n",
    "                          # Roll rate in rad/s\n",
    "                          'p': 0.0 / 180.0 * np.pi,\n",
    "                          # Pitch rate in rad/s\n",
    "                          'q': 0.0 / 180.0 * np.pi,\n",
    "                          # Yaw rate in rad/s\n",
    "                          'r': 0.0,\n",
    "                          # Roll acceleration in rad/s^2\n",
    "                          'pdot': 0.0 ,\n",
    "                          # Pitch acceleration in rad/s^2\n",
    "                          'qdot': 0.0,\n",
    "                          # Yaw acceleration in rad/s^2\n",
    "                          'rdot': 0.0,\n",
    "                          },\n",
    "                         {'desc': 'CC.M3.OVCFL000.pushdown',  # Descriptive string of the maneuver case\n",
    "                          # Kind of trim condition, blank for trim about all three axes, for more trim conditions see\n",
    "                          # trim_conditions.py\n",
    "                          'maneuver': '',\n",
    "                          # Subcase ID number, for Nastran in acending order\n",
    "                          'subcase': 2,\n",
    "                          # Setting of the operational point\n",
    "                          # The flight speed is given by the Mach number\n",
    "                          'Ma': tas2Ma(70.0, 0.0),\n",
    "                          # Aero key\n",
    "                          'aero': 'VC',\n",
    "                          # Atmo key\n",
    "                          'altitude': 'FL000',\n",
    "                          # Mass key\n",
    "                          'mass': 'M3',\n",
    "                          # Load factor Nz\n",
    "                          'Nz': -1.0,\n",
    "                          # Velocities and accelerations given in ISO 9300 coordinate system (right-handed, forward-right-down)\n",
    "                          # Roll rate in rad/s\n",
    "                          'p': 0.0 / 180.0 * np.pi,\n",
    "                          # Pitch rate in rad/s\n",
    "                          'q': 0.0 / 180.0 * np.pi,\n",
    "                          # Yaw rate in rad/s\n",
    "                          'r': 0.0,\n",
    "                          # Roll acceleration in rad/s^2\n",
    "                          'pdot': 0.0,\n",
    "                          # Pitch acceleration in rad/s^2\n",
    "                          'qdot': 0.0,\n",
    "                          # Yaw acceleration in rad/s^2\n",
    "                          'rdot': 0.0,\n",
    "                          },\n",
    "                         {'desc': 'CC.M3.OVCFL000.pullup',  # Descriptive string of the maneuver case\n",
    "                          # Kind of trim condition, blank for trim about all three axes, for more trim conditions see\n",
    "                          # trim_conditions.py\n",
    "                          'maneuver': '',\n",
    "                          # Subcase ID number, for Nastran in acending order\n",
    "                          'subcase': 3,\n",
    "                          # Setting of the operational point\n",
    "                          # The flight speed is given by the Mach number\n",
    "                          'Ma': tas2Ma(70.0, 0.0),\n",
    "                          # Aero key\n",
    "                          'aero': 'VC',\n",
    "                          # Atmo key\n",
    "                          'altitude': 'FL000',\n",
    "                          # Mass key\n",
    "                          'mass': 'M3',\n",
    "                          # Load factor Nz\n",
    "                          'Nz': 2.5,\n",
    "                          # Velocities and accelerations given in ISO 9300 coordinate system (right-handed, forward-right-down)\n",
    "                          # Roll rate in rad/s\n",
    "                          'p': 0.0,\n",
    "                          # Pitch rate in rad/s\n",
    "                          'q': 0.0,\n",
    "                          # Yaw rate in rad/s\n",
    "                          'r': 0.0,\n",
    "                          # Roll acceleration in rad/s^2\n",
    "                          'pdot': 0.0,\n",
    "                          # Pitch acceleration in rad/s^2\n",
    "                          'qdot': 0.0,\n",
    "                          # Yaw acceleration in rad/s^2\n",
    "                          'rdot': 0.0,\n",
    "                          }]"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "17aceb3a-138c-4958-8a2a-393ec6d10bc3",
   "metadata": {},
   "source": [
    "For every trimcase, a corresponding simcase is required. For maneuvers, it may be empty self.simcase = [{}]."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "957e3b74-7d52-4720-899c-85030d8453cf",
   "metadata": {},
   "outputs": [],
   "source": [
    "class jcl:\n",
    "\n",
    "    def __init__(self):\n",
    "\n",
    "        self.simcase  = jcl_helper.generate_empty_listofdicts(self.trimcase)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "5e31f5d9-6185-4c3f-b316-bdda8f244083",
   "metadata": {},
   "source": [
    "After completing the set-up, the 'launch.py' file (found in tutorials -> DC3_model -> JCLs) is run. The name of the file, as well as the input and output file locations are specified. The results can be found in the folder 'DC-3_results'."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "6b761e81-72b9-4260-872d-1727fbe39099",
   "metadata": {
    "tags": [
     "skip-execution"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "INFO: This is the log for process 0.\n",
      "INFO: Starting Loads Kernel with job: jcl_dc3_trim\n",
      "INFO: User carn_fr on schwalbe (Linux-4.18.0-513.24.1.el8_9.x86_64-x86_64-with-glibc2.28)\n",
      "INFO: pre:  True\n",
      "INFO: main: True\n",
      "INFO: post: True\n",
      "INFO: test: False\n",
      "INFO: --> Reading parameters from JCL.\n",
      "INFO: Generated list of 3 empty dicts.\n",
      "INFO: --> Starting preprocessing.\n",
      "INFO: Building structural model...\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/structure_only.bdf\n",
      "INFO: Found include(s):\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/export_FUS.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-wing/left-wing.GRID_LREFAX_5400001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-wing/left-wing.RBE2_LREFAX_5400001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-wing/left-wing.CORD2R_LREFAX\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-wing/left-wing.MAT_ZR\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-wing/export_left-wing.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-wing/right-wing.GRID_LREFAX_6400001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-wing/right-wing.RBE2_LREFAX_6400001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-wing/right-wing.CORD2R_LREFAX\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-wing/right-wing.MAT_ZR\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-wing//export_right-wing.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-ht/left-ht.GRID_LREFAX_3330001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-ht/left-ht.RBE2_LREFAX_3330001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-ht/left-ht.CORD2R_LREFAX\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-ht/left-ht.MAT_ZR\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/left-ht/export_left-ht.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-ht/right-ht.GRID_LREFAX_3340001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-ht/right-ht.RBE2_LREFAX_3340001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-ht/right-ht.CORD2R_LREFAX\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-ht/right-ht.MAT_ZR\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/right-ht/export_right-ht.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/vt/vt.GRID_LREFAX_3320001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/vt/vt.RBE2_LREFAX_3320001\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/vt/vt.CORD2R_LREFAX\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/vt/vt.MAT_ZR\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/vt/export_vt.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/export_left-nacell.csv\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/../fem/export_right-nacell.csv\n",
      "INFO: The structural model consists of 278 grid points (1668 DoFs) and 7 coordinate systems.\n",
      "INFO: Reading Monitoring Stations from MONPNTs...\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/export_monitoring-stations.csv\n",
      "INFO: Splining (rigid body) of 1668 DOFs to 192 DOFs...\n",
      "INFO: Building atmo model...\n",
      "INFO: Building aero model...\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/vt/vt.CAERO1\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/left-ht/left-ht.CAERO1\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/right-ht/right-ht.CAERO1\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/left-wing/left-wing.CAERO1\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/right-wing/right-wing.CAERO1\n",
      "INFO: Constructing aero panels from CAERO cards\n",
      "INFO:  - from corner points and aero panels, constructing aerogrid\n",
      "INFO: Read W2GJ data from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/w2gj_list.DMI_merge\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6336 DOFs...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6336 DOFs...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6 DOFs...\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/vt/vt.AESURF\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/left-ht/left-ht.AESURF\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/right-ht/right-ht.AESURF\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/left-wing/left-wing.AESURF\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/right-wing/right-wing.AESURF\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/vt/vt.AELIST\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/left-ht/left-ht.AELIST\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/right-ht/right-ht.AELIST\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/left-wing/left-wing.AELIST\n",
      "INFO: Read from file: /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/aero/right-wing/right-wing.AELIST\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 6 DOFs...\n",
      "INFO: Calculating steady AIC matrices (1056 panels, k=0.0) for 2 Mach number(s)...\n",
      "INFO: done in 1.03 [sec].\n",
      "INFO: The aerodynamic model consists of 1056 panels and 5 control surfaces.\n",
      "INFO: Coupling aerogrid directly. Doing cleanup/thin out of strcgrid to avoid singularities                     (safety first!)\n",
      "INFO: The spline model consists of 272 grid points.\n",
      "INFO: Searching nearest neighbour of 1056 dependent nodes in 272 independent nodes...\n",
      "INFO: Splining (rigid body) of 6336 DOFs to 1668 DOFs...\n",
      "INFO: Building stiffness and mass model...\n",
      "INFO: Reading matrix KGG from /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/SOL103_structure_only.mtx.h5\n",
      "INFO: Reading matrix GM from /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/SOL103_structure_only.mtx.h5\n",
      "INFO: Read USET from OP2-file /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/uset.op2 ...\n",
      "Table  USET    , bytes =       6935 [         0 to       6936]\n",
      "INFO: Extracting bit positions from USET to determine DoFs\n",
      "INFO: Prepare stiffness matrices for independent and free DoFs (f-set)\n",
      "INFO: Mass configuration 1 of 1: M3 \n",
      "INFO: Reading matrix MGG from /data/carn_fr/LoadsKernel/doc/tutorials/DC3_model/fem/SOL103_M3.mtx.h5\n",
      "INFO: Calculate center of gravity, mass and inertia (GRDPNT)...\n",
      "INFO: Splining (rigid body) of 1668 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 1668 DOFs to 6 DOFs...\n",
      "INFO: Mass: 11883.983000000002\n",
      "INFO: CG at: [8.62280419e+00 5.43474355e-08 3.11704131e-01]\n",
      "INFO: Inertia: \n",
      "[[ 6.93201323e+04 -3.80956656e-04  1.17729353e+04]\n",
      " [-3.80956654e-04  1.40925492e+05 -1.74794555e-04]\n",
      " [ 1.17729353e+04 -1.74794555e-04  1.97104534e+05]]\n",
      "INFO: Prepare mass matrices for independent and free DoFs (f-set)\n",
      "INFO: Modal analysis for first 76 modes...\n",
      "INFO: Found 76 modes with the following frequencies [Hz]:\n",
      "INFO: [           nan            nan 1.05457802e-06 4.52538930e-06\n",
      " 1.45457362e-05 3.39333114e-05 3.13716140e+00 4.68251648e+00\n",
      " 7.20798842e+00 7.88159198e+00 8.33703334e+00 8.49130434e+00\n",
      " 9.88499185e+00 1.25695152e+01 1.53520000e+01 1.70224904e+01\n",
      " 1.71353125e+01 1.84415883e+01 2.53323405e+01 2.53529787e+01\n",
      " 2.68433851e+01 2.81886245e+01 3.20724571e+01 3.24562324e+01\n",
      " 3.51081211e+01 3.52877859e+01 3.71484020e+01 3.74387461e+01\n",
      " 4.65087795e+01 4.66555395e+01 5.19704569e+01 5.31897897e+01\n",
      " 5.59827047e+01 5.61588818e+01 5.79161872e+01 5.83859187e+01\n",
      " 6.49254027e+01 6.49636994e+01 6.52953993e+01 6.55893563e+01\n",
      " 6.76977070e+01 7.39272939e+01 7.40020584e+01 7.92493678e+01\n",
      " 7.98448917e+01 8.44686963e+01 8.44977700e+01 8.89696420e+01\n",
      " 8.90380160e+01 9.17266784e+01 9.27516791e+01 1.00718906e+02\n",
      " 1.00742526e+02 1.05800840e+02 1.06520421e+02 1.09898735e+02\n",
      " 1.10071029e+02 1.12603325e+02 1.12677121e+02 1.21628942e+02\n",
      " 1.21633308e+02 1.25797288e+02 1.25826624e+02 1.30369009e+02\n",
      " 1.30516978e+02 1.37566261e+02 1.37608584e+02 1.41676350e+02\n",
      " 1.44303157e+02 1.44811563e+02 1.47038007e+02 1.47151873e+02\n",
      " 1.55547154e+02 1.56081100e+02 1.56183006e+02 1.57181628e+02]\n",
      "INFO: From these 76 modes, the following 70 modes are selected: [ 7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30\n",
      " 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54\n",
      " 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76]\n",
      "INFO: Working on f-set\n",
      "INFO: Damping: modal damping of 0.02\n",
      "INFO: Working on h-set\n",
      "INFO: Splining (rigid body) of 1668 DOFs to 6 DOFs...\n",
      "INFO: Damping: modal damping of 0.02\n",
      "INFO: Splining (rigid body) of 1668 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6 DOFs to 6 DOFs...\n",
      "INFO: Splining (rigid body) of 6 DOFs to 6 DOFs...\n",
      "INFO: --> Saving model data.\n",
      "INFO: --> Done in 0:00:04 [h:mm:ss].\n",
      "INFO: --> Starting Main in sequential mode for 3 trimcase(s).\n",
      "INFO: \n",
      "INFO: ========================================\n",
      "INFO: trimcase: CC.M3.OVCFL000.level\n",
      "INFO: subcase: 1\n",
      "INFO: (case 1 of 3)\n",
      "INFO: ========================================\n",
      "INFO: Setting trim conditions to \"default\"\n",
      "INFO: Init model equations of type \"Steady\"\n",
      "INFO: Init EFCS \"efcs_dc3\"\n",
      "INFO: Running trim for 76 variables...\n",
      "INFO: The solution converged.\n",
      "INFO: calculating forces & moments on structural set (force summation method)...\n",
      "INFO: apply euler angles...\n",
      "INFO: calculating cutting forces & moments...\n",
      "INFO: gathering information on monitoring stations from response(s)...\n",
      "INFO: searching min/max in time data at 32 monitoring stations and gathering loads (dyn2stat)...\n",
      "INFO: --> Saving response(s).\n",
      "INFO: \n",
      "INFO: ========================================\n",
      "INFO: trimcase: CC.M3.OVCFL000.pushdown\n",
      "INFO: subcase: 2\n",
      "INFO: (case 2 of 3)\n",
      "INFO: ========================================\n",
      "INFO: Setting trim conditions to \"default\"\n",
      "INFO: Init model equations of type \"Steady\"\n",
      "INFO: Init EFCS \"efcs_dc3\"\n",
      "INFO: Running trim for 76 variables...\n",
      "INFO: The solution converged.\n",
      "INFO: calculating forces & moments on structural set (force summation method)...\n",
      "INFO: apply euler angles...\n",
      "INFO: calculating cutting forces & moments...\n",
      "INFO: gathering information on monitoring stations from response(s)...\n",
      "INFO: searching min/max in time data at 32 monitoring stations and gathering loads (dyn2stat)...\n",
      "INFO: --> Saving response(s).\n",
      "INFO: \n",
      "INFO: ========================================\n",
      "INFO: trimcase: CC.M3.OVCFL000.pullup\n",
      "INFO: subcase: 3\n",
      "INFO: (case 3 of 3)\n",
      "INFO: ========================================\n",
      "INFO: Setting trim conditions to \"default\"\n",
      "INFO: Init model equations of type \"Steady\"\n",
      "INFO: Init EFCS \"efcs_dc3\"\n",
      "INFO: Running trim for 76 variables...\n",
      "INFO: The solution converged.\n",
      "INFO: calculating forces & moments on structural set (force summation method)...\n",
      "INFO: apply euler angles...\n",
      "INFO: calculating cutting forces & moments...\n",
      "INFO: gathering information on monitoring stations from response(s)...\n",
      "INFO: searching min/max in time data at 32 monitoring stations and gathering loads (dyn2stat)...\n",
      "INFO: --> Saving response(s).\n",
      "INFO: --> Saving monstation(s).\n",
      "INFO: --> Saving dyn2stat.\n",
      "INFO: --> Done in 0:00:02 [h:mm:ss].\n",
      "INFO: --> Opening response(s).\n",
      "INFO: --> Loading monstations(s).\n",
      "INFO: --> Loading dyn2stat.\n",
      "INFO: --> Drawing some standard plots.\n",
      "INFO: start potato-plotting...\n",
      "INFO: start plotting cutting forces along wing...\n",
      "INFO: plots saved as ./DC3_results/monstations_jcl_dc3_trim.pdf\n",
      "INFO: --> Saving auxiliary output data.\n",
      "INFO: writing trim results to: ./DC3_results/trim_results_jcl_dc3_trim.csv\n",
      "INFO: writing successful trimcases cases to: ./DC3_results/successful_trimcases_jcl_dc3_trim.csv\n",
      "INFO: writing failed trimcases cases to: ./DC3_results/failed_trimcases_jcl_dc3_trim.csv\n",
      "INFO: writing critical trimcases cases to: ./DC3_results/crit_trimcases_jcl_dc3_trim.csv\n",
      "INFO: saving critical nodal loads as Nastarn cards...\n",
      "INFO: Loads Kernel finished.\n",
      "INFO: \n",
      "INFO:        (  )\n",
      "INFO:       (    )\n",
      "INFO: \n",
      "INFO:               (   )\n",
      "INFO:              (     )\n",
      "INFO: \n",
      "INFO:          _|_\n",
      "INFO:  ---------O---------\n",
      "INFO: \n",
      "INFO: \n"
     ]
    }
   ],
   "source": [
    "from loadskernel import program_flow\n",
    "\n",
    "# Here you launch the Loads Kernel with your job\n",
    "k = program_flow.Kernel('jcl_dc3_trim', pre=True, main=True, post=True, test=False,\n",
    "                        path_input='./DC3_model/JCLs',\n",
    "                        path_output='./DC3_results')\n",
    "k.run()"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.13"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
